# Copyright 1999-2015. Parallels IP Holdings GmbH. All Rights Reserved.
# -*- coding: utf-8; mode: python -*-
"""This proto-module contains various constants and functions
common for both pam_plesk_install and pam_plesk_remove"""

import sys
# FIXME: should we put getopt under an import-error safe wrapper?
import getopt

##try: import logging
##except ImportError: import compat.dummy_logging as logging
import logging

from traceback import format_exception, format_exception_only

try: from optparse import OptionParser
except ImportError: from compat.optparse import OptionParser

try: from optparse import make_option
except ImportError: from compat.optparse import make_option


from PAMConfig.defaults import ConfigurationFile as confFile
from PAMConfig.defaults import ConfigurationDirectory as confDir

# default [settings] section
PAMPleskModule         = "pam_plesk.so"

PAMPleskCryptedService = "plesk_usermng_encrypted"
PAMPleskPlainService   = "passwd"

def parseArgs(optadd=[]):
    optlist = [
        make_option("-f", "--file", dest="confFile", metavar="FILE",
                    default=confFile,
                    help="path to PAM conf. file (%s by default)" % confFile),
        
        make_option("-d", "--dir", dest="confDir", metavar="DIR",
                    default=confDir,
                    help="path to PAM conf. directory (%s by default)" % confDir),
        
        make_option("-m", "--module", dest="pleskModule", metavar="FILE",
                    default=PAMPleskModule,
                    help="PAM Plesk module name or path (%s by default)" % PAMPleskModule),
        
        make_option("-l", "--install-log", dest="installLogFile", metavar="FILE",
                    help="path to regular installation log file"),
        
        make_option("-e", "--problem-log", dest="problemLogFile", metavar="FILE",
                    help="path to problem report file"),
        
        make_option("-v", "--verbose", dest="verbosity", action="count", default=0,
                    help="increase number of logging messages"),
        
        make_option("-q", "--quiet", dest="reticence", action="count", default=0,
                    help="decrease number of logging messages")
        
        ]

    optlist.extend(optadd)
    parser = OptionParser(option_list=optlist)
        
    (options, args) = parser.parse_args()

    options.logLevel = options.reticence - options.verbosity

    return (options, args)
# enddef parseArgs

def setupLoggers(options):
    logger = logging.getLogger() # root logger
    logger.setLevel(logging.DEBUG)
    
    shift = options.logLevel * 10
    baseLevel = logger.getEffectiveLevel() + shift

    if baseLevel <= 0:
        baseLevel = logging.DEBUG

    cl = logging.StreamHandler() # sys.stderr by default
    cl.setFormatter(logging.Formatter("%(levelname)s:%(message)s"))
    consoleLevel = logging.CRITICAL # by default be quite picky

    installLogFile = getattr(options, 'installLogFile', None)
    if installLogFile:
        lf = logging.FileHandler(installLogFile)
        lf.setLevel(logging.INFO)
        logger.addHandler(lf)
    else:
        # console should handle informational messages as well
        consoleLevel = logging.INFO
    # endif

    problemLogFile = getattr(options, 'problemLogFile', None)
    if problemLogFile:
        pf = logging.FileHandler(problemLogFile)
        pf.setLevel(baseLevel)
        logger.addHandler(pf)
    else:
        consoleLevel = baseLevel
    # endif

    cl.setLevel(consoleLevel)
    logger.addHandler(cl)
# enddef setupLoggers
    
def exitmsg(code, msg):
    if code:
        logging.critical("%s:%s" % (code, msg))
    sys.exit(code)
# enddef exitmsg

def sortChanges(changes):
    # now sort them in reversed order so a change made to a service
    # won't disturb others... Mostly it's needed for FreeBSD-4.x
    # with a single /etc/pam.conf (other systems permit this PAM configuration
    # style too but /etc/pam.d/ is more preferred)
    #
    changes.sort(lambda t1, t2: cmp(t1[1], t2[1])) # in-place sort but anyway
    return changes
# enddef sortChanges

def formatExceptionShort(extype, exvalue):
    return ''.join(format_exception_only(extype, exvalue)).strip()
# enddef formatExceptionShort

